@@ -4,12 +4,12 @@ import sys
 from abc import ABCMeta
 from collections import defaultdict
 from collections.abc import Hashable
-from typing import List, Type, Optional, TypeVar, Union, Generic
+from typing import List, Type, Optional, TypeVar, Union, Generic, DefaultDict
 
 import inflection
 import uritemplate
 from django import __version__ as DJANGO_VERSION
-from django.urls.resolvers import _PATH_PARAMETER_COMPONENT_RE, get_resolver
+from django.urls.resolvers import _PATH_PARAMETER_COMPONENT_RE, get_resolver  # type: ignore
 from django.utils.module_loading import import_string
 from rest_framework import fields, serializers, versioning, exceptions
 
@@ -27,8 +27,8 @@ class UnableToProceedError(Exception):
 
 
 class GeneratorStats:
-    _warn_cache = defaultdict(int)
-    _error_cache = defaultdict(int)
+    _warn_cache: DefaultDict[str, int] = defaultdict(int)
+    _error_cache: DefaultDict[str, int] = defaultdict(int)
 
     def __bool__(self):
         return bool(self._warn_cache or self._error_cache)
@@ -74,7 +74,7 @@ def anyisinstance(obj, type_list):
     return any([isinstance(obj, t) for t in type_list])
 
 
-def get_class(obj):
+def get_class(obj) -> type:
     return obj if inspect.isclass(obj) else obj.__class__
 
 
@@ -214,6 +214,10 @@ def safe_ref(schema):
     return schema
 
 
+def append_meta(schema, meta):
+    return safe_ref({**schema, **meta})
+
+
 def get_field_from_model(model, field):
     """
     this is a Django 2.2 compatibility function to access a field through a Deferred Attribute
@@ -349,7 +353,7 @@ class ComponentRegistry:
         del self._components[key]
 
     def build(self, extra_components) -> dict:
-        output = defaultdict(dict)
+        output: DefaultDict[str, dict] = defaultdict(dict)
         # build tree from flat registry
         for component in self._components.values():
             output[component.type][component.name] = component.schema
@@ -357,14 +361,10 @@ class ComponentRegistry:
         for extra_type, extra_component_dict in extra_components.items():
             for component_name, component_schema in extra_component_dict.items():
                 output[extra_type][component_name] = component_schema
-
-        if 'schemas' in output:
-            postprocess_schema_enums(output['schemas'])
-
         # sort by component type then by name
         return {
-            type: {name: output[type][name] for name in output[type].keys()}
-            for type in sorted(output.keys(), reverse=True)
+            type: {name: output[type][name] for name in sorted(output[type].keys())}
+            for type in sorted(output.keys())
         }
 
 
@@ -372,6 +372,7 @@ class OpenApiGeneratorExtension(Generic[T], metaclass=ABCMeta):
     _registry: List[T] = []
     target_class: Union[None, str, Type[object]] = None
     match_subclasses = False
+    priority = 0
 
     def __init_subclass__(cls, **kwargs):
         super().__init_subclass__(**kwargs)
@@ -395,19 +396,19 @@ class OpenApiGeneratorExtension(Generic[T], metaclass=ABCMeta):
         if cls.target_class is None:
             return False  # app not installed
         elif cls.match_subclasses:
-            return issubclass(get_class(target), cls.target_class)
+            return issubclass(get_class(target), cls.target_class)  # type: ignore
         else:
             return get_class(target) == cls.target_class
 
     @classmethod
     def get_match(cls, target) -> Optional[T]:
-        for extension in cls._registry:
+        for extension in sorted(cls._registry, key=lambda e: e.priority, reverse=True):
             if extension._matches(target):
                 return extension(target)
         return None
 
 
-def postprocess_schema_enums(schemas):
+def postprocess_schema_enums(result, **kwargs):
     """
     simple replacement of Enum/Choices that globally share the same name and have
     the same choices. Aids client generation to not generate a separate enum for
@@ -423,6 +424,9 @@ def postprocess_schema_enums(schemas):
             yield from iter_prop_containers(schema.get('oneOf', []))
             yield from iter_prop_containers(schema.get('allOf', []))
 
+    # schemas will be modified in-place and same result object is returned
+    schemas = result.get('components', {}).get('schemas', {})
+
     hash_mapping = defaultdict(set)
     # collect all enums, their names and contents
     for props in iter_prop_containers(list(schemas.values())):
@@ -453,6 +457,8 @@ def postprocess_schema_enums(schemas):
             schemas[enum_name] = enum_schema
             props[prop_name] = safe_ref(prop_schema)
 
+    return result
+
 
 def resolve_regex_path_parameter(path_regex, variable):
     """
