@@ -13,6 +13,12 @@ SPECTACULAR_DEFAULTS: Dict[str, Any] = {
     'OPERATION_SORTER': 'alpha',
     'DEFAULT_GENERATOR_CLASS': 'drf_spectacular.generators.SchemaGenerator',
 
+    # Schema generation parameters to influence how components are constructed.
+    # Some schema features might not translate well to your target.
+    # Demultiplexing components might help alleviate those issues.
+    'COMPONENT_SPLIT_PATCH': True,
+    'COMPONENT_SPLIT_REQUEST': False,
+
     # Configuration for serving the schema with SpectacularAPIView
     'SERVE_URLCONF': None,
     # complete public schema or a subset based on the requesting user
@@ -25,6 +31,12 @@ SPECTACULAR_DEFAULTS: Dict[str, Any] = {
     'APPEND_PATHS': {},
     'APPEND_COMPONENTS': {},
 
+    # Postprocessing functions that run at the end of schema generation.
+    # must satisfy interface result = hook(generator, request, public, result)
+    'POSTPROCESSING_HOOKS': [
+        'drf_spectacular.plumbing.postprocess_schema_enums'
+    ],
+
     # General schema metadata. Refer to spec for valid inputs
     # https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.0.3.md#openapi-object
     'TITLE': '',
@@ -57,6 +69,7 @@ IMPORT_STRINGS = [
     'SCHEMA_AUTHENTICATION_CLASSES',
     'DEFAULT_GENERATOR_CLASS',
     'SERVE_PERMISSIONS',
+    'POSTPROCESSING_HOOKS',
 ]
 
 spectacular_settings = APISettings(
